/*
 * Copyright 1993-2011 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO LICENSEE:
 *
 * This source code and/or documentation ("Licensed Deliverables") are
 * subject to NVIDIA intellectual property rights under U.S. and
 * international Copyright laws.
 *
 * These Licensed Deliverables contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and
 * conditions of a form of NVIDIA software license agreement by and
 * between NVIDIA and Licensee ("License Agreement") or electronically
 * accepted by Licensee.  Notwithstanding any terms or conditions to
 * the contrary in the License Agreement, reproduction or disclosure
 * of the Licensed Deliverables to any third party without the express
 * written consent of NVIDIA is prohibited.
 *
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE
 * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  IT IS
 * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND.
 * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED
 * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY
 * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THESE LICENSED DELIVERABLES.
 *
 * U.S. Government End Users.  These Licensed Deliverables are a
 * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT
 * 1995), consisting of "commercial computer software" and "commercial
 * computer software documentation" as such terms are used in 48
 * C.F.R. 12.212 (SEPT 1995) and is provided to the U.S. Government
 * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and
 * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all
 * U.S. Government End Users acquire the Licensed Deliverables with
 * only those rights set forth herein.
 *
 * Any use of the Licensed Deliverables in individual and commercial
 * software must include, in the user documentation and internal
 * comments to the code, the above Disclaimer and U.S. Government End
 * Users Notice.
 */

#if !defined(__CUPTI_CALLBACKS_H__)
#define __CUPTI_CALLBACKS_H__

/* Driver api types */
#include <cuda.h>

/* Runtime api types */
#include <builtin_types.h>         

/* define size_t */
#include <string.h>

/* define standard integer types */
#include <cuda_stdint.h>

/* CUPTI result codes */
#include <cupti_result.h>

#ifndef CUPTIAPI
#ifdef _WIN32
#define CUPTIAPI __stdcall
#else 
#define CUPTIAPI
#endif 
#endif

#if defined(__cplusplus)
extern "C" {
#endif 

/* -------------------------------------------------------------------------- */

/**
 * \defgroup CUPTI_CALLBACK_API CUPTI Callback API
 * @{
 */

/**
 * \brief Specifies the point in an API call that a callback is issued.
 *
 * Specifies the point in an API call that a callback is issued. This
 * value is communicated to the callback function via \ref
 * CUpti_CallbackData::callbackSite.
 */
typedef enum {
    /**
     * The callback is at the entry of the API call.
     */
    CUPTI_API_ENTER                 = 0,
    /** 
     * The callback is at the exit of the API call.
     */
    CUPTI_API_EXIT                  = 1,
    CUPTI_API_CBSITE_FORCE_INT     = 0x7fffffff
} CUpti_ApiCallbackSite;

/**
 * \brief Callback domains.
 *
 * Callback domains. Each domain represents callback points for a
 * group of related API functions.
 */
typedef enum {
    /**
     * Invalid domain.
     */
    CUPTI_CB_DOMAIN_INVALID           = 0,
    /**
     * Domain containing callback points for all driver API functions.
     */
    CUPTI_CB_DOMAIN_DRIVER_API        = 1,
    /**
     * Domain containing callback points for all runtime API functions.
     */
    CUPTI_CB_DOMAIN_RUNTIME_API       = 2,
    CUPTI_CB_DOMAIN_FORCE_INT         = 0x7fffffff
} CUpti_CallbackDomain;

/**
 * \brief Data passed into a runtime or driver API callback function.
 *
 * Data passed into a runtime or driver API callback function as the
 * \p cbdata argument to \ref CUpti_CallbackFunc. The callback data is
 * valid only within the invocation of the callback function that is
 * passed the data. If you need to retain some data for use outside of
 * the callback, you must make a copy of that data. For example, if
 * you make a shallow copy of CUpti_CallbackData within a callback,
 * you cannot dereference \p functionParams outside of that callback
 * to access the function parameters. \p functionName is an exception:
 * the string pointed to by \p functionName is a global constant and
 * so may be accessed in any context.
 */
typedef struct {
    /**
     * Point in the runtime or driver function from where the callback
     * was issued.
     */ 
    CUpti_ApiCallbackSite callbackSite;

    /**
     * Name of the runtime or driver API function which issued the
     * callback.
     */ 
    const char *functionName;

    /**  
     * Pointer to the arguments passed to the runtime or driver API
     * call. See \ref generated_cuda_runtime_api_meta.h and \ref
     * generated_cuda_meta.h for structure definitions for the
     * parameters for each runtime and driver API function.
     */
    const void *functionParams;

    /**
     * Pointer to the return value of the runtime or driver API
     * call. This field is only valid within the exit::CUPTI_API_EXIT
     * callback. For a runtime API \p functionReturnValue points to a
     * \p cudaError_t. For a driver API \p functionReturnValue points
     * to a \p CUresult.
     */
    void *functionReturnValue;

    /**
     * Name of the symbol operated on by the runtime or driver API
     * function which issued the callback. This entry is valid only
     * for the runtime cudaLaunch callback
     * (i.e. CUPTI_RUNTIME_TRACE_CBID_cudaLaunch_v3020), where it
     * returns the name of the kernel.
     */ 
    const char *symbolName;

    /** 
     * Driver context current to the thread, or null if no context is
     * current. This value can change from the entry to exit callback
     * of a runtime API function if the runtime initializes a context.
     */
    CUcontext context;
    
    /**
     * Unique ID for the CUDA context associated with the thread. The
     * UIDs are assigned sequentially as contexts are created and are
     * unique within a process. A UID of zero indicates that there is
     * no context current to the thread, or that a context has not yet
     * been attached to the thread during runtime-driver interop.
     */
    uint64_t contextUid;  

    /**
     * Pointer to data shared between the entry and exit callbacks of
     * a given runtime or drive API function invocation. This field
     * can be used to pass 64-bit values from the entry callback to
     * the corresponding exit callback.
     */
    uint64_t *correlationData;
 
} CUpti_CallbackData;

/**
 * \brief Pointer to an array of callback domains.
 */
typedef CUpti_CallbackDomain *CUpti_DomainTable;

/**
 * \brief An ID for a driver or runtime API function.
 *
 * An ID for a driver or runtime API function. Within a driver API
 * callback this should be interpreted as a \ref
 * CUpti_driver_api_trace_cbid value. Within a runtime API callback
 * this should be interpreted as a \ref CUpti_runtime_api_trace_cbid
 * value.
 */ 
typedef uint32_t CUpti_CallbackId;

/**
 * \brief Function type for an API callback
 *
 * Function type for an API callback
 *
 * \param userdata User data supplied at subscription of the callback
 * \param domain The domain of the callback
 * \param cbid The ID of the API function associated with this callback
 * \param cbdata Data passed to the callback.
 */
typedef void (CUPTIAPI *CUpti_CallbackFunc)(
    void *userdata,
    CUpti_CallbackDomain domain,
    CUpti_CallbackId cbid,
    const CUpti_CallbackData *cbdata); 

/**
 * \brief A callback subscriber.
 */
typedef struct CUpti_Subscriber_st *CUpti_SubscriberHandle;

/* ------------------------------------------------------------------------- */

/**
 * \brief Get the available callback domains.
 * 
 * Returns in \p *domainTable an array of size \p *domainCount of all
 * the available callback domains. A callback domain is a set of
 * callback points for a related group of API functions.
 * \note \b Thread-safety: this function is thread safe.
 *
 * \param domainCount Returns number of callback domains
 * \param domainTable Returns pointer to array of available callback domains 
 * 
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialize CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p domainCount or \p domainTable are NULL
 */
CUptiResult CUPTIAPI cuptiSupportedDomains(size_t *domainCount, 
					   CUpti_DomainTable *domainTable);

/** 
 * \brief Initialize a callback subscriber with a callback function
 * and user data.
 * 
 * Initializes a callback subscriber with a callback function and
 * (optionally) a pointer to user data. The returned subscriber handle
 * can be used to enable and disable the callback for specific domains
 * and API functions.  data can also be provided.
 * \note This function does not enable any callbacks.
 * \note \b Thread-safety: this function is thread safe.
 *
 * \param subscriber Returns handle to initialize subscriber
 * \param callback The callback function
 * \param userdata A pointer to user data. This data will be passed to
 * the callback function via the \p userdata paramater.
 *
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_OUT_OF_MEMORY
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialize CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p subscriber is NULL
 * \retval CUPTI_ERROR_UNKNOWN if the subscriber limit is reached
 */
CUptiResult CUPTIAPI cuptiSubscribe(CUpti_SubscriberHandle *subscriber, 
				    CUpti_CallbackFunc callback, 
				    void *userdata);

/**
 * \brief Unregister a callback subscriber.
 * 
 * Removes a callback subscriber so that no future callbacks will be
 * issued to that subscriber.
 * \note \b Thread-safety: this function is thread safe.
 *
 * \param subscriber Handle to the initialize subscriber
 *
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialized CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p subscriber is NULL or not initialized
 */
CUptiResult CUPTIAPI cuptiUnsubscribe(CUpti_SubscriberHandle subscriber); 

/**
 * \brief Get the current enabled/disabled state of a callback for a specific
 * domain and function ID. 
 * 
 * Returns non-zero in \p *enable if the callback for a domain and API
 * function is enabled, and zero if not enabled.
 *
 * \note \b Thread-safety: function is thread safe for different
 * subscribers, but each subscriber must serialize access to
 * cuptiGetCallbackState, cuptiEnableCallback, cuptiEnableDomain, and
 * cuptiEnableAllDomains. For example, if cuptiGetCallbackEnabled(sub,
 * d, c) and cuptiEnableCallback(sub, d, c) are called concurrently,
 * the results are undefined.
 * 
 * \param enable Returns non-zero if callback enabled, zero if not enabled
 * \param subscriber Handle to the initialize subscriber
 * \param domain The domain of the callback
 * \param cbid The ID of the API function
 *
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialized CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p enabled is NULL, or if \p
 * subscriber, \p domain or \p cbid is invalid.
 */
CUptiResult CUPTIAPI cuptiGetCallbackState(uint32_t *enable,               
					   CUpti_SubscriberHandle subscriber,
					   CUpti_CallbackDomain domain,
					   CUpti_CallbackId cbid);

/**
 * \brief Enable or disabled callbacks for a specific domain and
 * function ID.
 *
 * Enable or disabled callbacks for a subscriber for a specific domain
 * and function ID.
 *
 * \note \b Thread-safety: function is thread safe for different
 * subscribers, but each subscriber must serialize access to
 * cuptiGetCallbackState, cuptiEnableCallback, cuptiEnableDomain, and
 * cuptiEnableAllDomains. For example, if cuptiGetCallbackEnabled(sub,
 * d, c) and cuptiEnableCallback(sub, d, c) are called concurrently,
 * the results are undefined.
 * 
 * \param enable New enable state for the callback. Zero disables the
 * callback, non-zero enables the callback.
 * \param subscriber - Handle to callback subscription 
 * \param domain The domain of the callback
 * \param cbid The ID of the API function
 *
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialized CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p subscriber, \p domain or \p
 * cbid is invalid.
 */
CUptiResult CUPTIAPI cuptiEnableCallback(uint32_t enable,
					 CUpti_SubscriberHandle subscriber, 
					 CUpti_CallbackDomain domain,
					 CUpti_CallbackId cbid);

/**
 * \brief Enable or disabled all callbacks for a specific domain.
 *
 * Enable or disabled all callbacks for a specific domain.
 *
 * \note \b Thread-safety: function is thread safe for different
 * subscribers, but each subscriber must serialize access to
 * cuptiGetCallbackState, cuptiEnableCallback, cuptiEnableDomain, and
 * cuptiEnableAllDomains. For example, if cuptiGetCallbackEnabled(sub,
 * d, *) and cuptiEnableDomain(sub, d) are called concurrently, the
 * results are undefined.
 *
 * \param enable New enable state for all callbacks in the
 * domain. Zero disables all callbacks, non-zero enables all
 * callbacks.
 * \param subscriber - Handle to callback subscription 
 * \param domain The domain of the callback
 *
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialized CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p subscriber or \p domain is invalid
 */
CUptiResult CUPTIAPI cuptiEnableDomain(uint32_t enable,
				       CUpti_SubscriberHandle subscriber, 
				       CUpti_CallbackDomain domain);

/**
 * \brief Enable or disable all callbacks in all domains.
 *
 * Enable or disable all callbacks in all domains.
 * 
 * \note \b Thread-safety: function is thread safe for different
 * subscribers, but each subscriber must serialize access to
 * cuptiGetCallbackState, cuptiEnableCallback, cuptiEnableDomain, and
 * cuptiEnableAllDomains. For example, if cuptiGetCallbackEnabled(sub,
 * d, *) and cuptiEnableAllDomains(sub) are called concurrently, the
 * results are undefined.
 *
 * \param enable New enable state for all callbacks in all
 * domain. Zero disables all callbacks, non-zero enables all
 * callbacks.
 * \param subscriber - Handle to callback subscription 
 *
 * \retval CUPTI_SUCCESS on success
 * \retval CUPTI_ERROR_NOT_INITIALIZED if unable to initialized CUPTI
 * \retval CUPTI_ERROR_INVALID_PARAMETER if \p subscriber is invalid
 */
CUptiResult CUPTIAPI cuptiEnableAllDomains(uint32_t enable, 
					   CUpti_SubscriberHandle subscriber);

/** @} */ /* END CUPTI_CALLBACK_API */ 

// ----------------------------------------------------------------------------
#if defined(__cplusplus)
}
#endif 

#endif  // file guard

